"""

"""

from pants.engine.target import COMMON_TARGET_FIELDS, Dependencies, MultipleSourcesField, Target
from pants.util.strutil import softwrap


class MdBookDependencies(Dependencies):
    alias = "dependencies"

    help = softwrap("""Dependencies to include in the build.""")


class MdBookSources(MultipleSourcesField):
    alias = "sources"
    default = ("book.toml", "src/*", "src/**/*")

    help = softwrap("""The sources to use for a MdBook book, including the book.toml.""")


class MdBook(Target):
    alias = "md_book"
    core_fields = (*COMMON_TARGET_FIELDS, MdBookSources, MdBookDependencies)

    help = softwrap("""A build target for a MdBook book. """)


def targets():
    return [
        MdBook,
    ]



"""

"""

from __future__ import annotations

from dataclasses import dataclass

from pants.engine.addresses import Address
from pants.engine.fs import Digest
from pants.engine.process import Process, ProcessResult
from pants.engine.rules import Get, collect_rules, rule

from pants_backend_mdbook.subsystem import MdBookTool
from pants_backend_mdbook.util_rules.prepare import MdBookAnalysis, MdBookAnalysisRequest


@dataclass(frozen=True)
class MdBookBuildOutput:
    digest: Digest | None


@dataclass(frozen=True)
class FallibleMdBookBuildOutput:
    success: bool

    output: MdBookBuildOutput | None = None


@dataclass(frozen=True)
class MdbookBuildRequest:
    address: Address


@rule(desc="Building MDBook")
async def build_mdbook(
    request: MdbookBuildRequest,
    mdbook: MdBookTool,
) -> FallibleMdBookBuildOutput:
    analysis = await Get(MdBookAnalysis, MdBookAnalysisRequest(request.address))
    result = await Get(
        ProcessResult,
        Process(
            input_digest=analysis.digest,
            argv=(analysis.tool_exe, "build", analysis.build_root),
            description=f"Building mdbook: {request.address}",
            output_directories=(f"{analysis.build_root}/book",),
        ),
    )

    return FallibleMdBookBuildOutput(
        success=True,
        output=MdBookBuildOutput(result.output_digest),
    )


def rules():
    return collect_rules()



"""

"""

from __future__ import annotations

from pants.core.util_rules.external_tool import ExternalTool
from pants.engine.platform import Platform

_mdbook_platform_mapping = {
    "linux_x86_64": "x86_64-unknown-linux-gnu",
    "linux_arm64": "aarch64-unknown-linux-musl",
    "macos_x86_64": "x86_64-apple-darwin",
    "macos_arm64": "x86_64-apple-darwin",
}


class MdBookTool(ExternalTool):
    options_scope = "mdbook"
    help = "The MdBook tool used to build static webpages."

    default_version = "v0.4.35"
    default_known_versions = [
        # using rosetta
        "v0.4.35|macos_arm64 |ca3281c2b5437a1ccd9079ed8121b3dd97c49be74dae32ea803b540a38c334bb|4701269",
        "v0.4.35|macos_x86_64|ca3281c2b5437a1ccd9079ed8121b3dd97c49be74dae32ea803b540a38c334bb|4701269",
        "v0.4.35|linux_x86_64|4ef777bfcb3fd01687deed990372a6eb5e125f79b592014b0ac09b61595f0b34|5348678",
        "v0.4.35|linux_arm64 |359af01b77fbd6bf6243a3f2b2491a37b5480bbb2674eb2d94f91354253b34f4|5442606",
        "v0.4.21|macos_x86_64|c1f3e8235f8b3128f6020397061c97444007e090ac42358402d3f25eee8499bd|4292102",
        "v0.4.21|linux_x86_64|ec3c978a255b444987fd6e0805147f4ea75d221f68c6e27dbf3e8a28aba166b7|4861607",
    ]

    def generate_url(self, plat: Platform) -> str:
        plat_str = _mdbook_platform_mapping[plat.value]
        base = "https://github.com/rust-lang/mdBook/releases/download"
        return f"{base}/{self.version}/mdbook-{self.version}-{plat_str}.tar.gz"

    def generate_exe(self, plat: Platform) -> str:
        return "./mdbook"


def rules():
    return [
        *MdBookTool.rules(),
    ]



import os
from dataclasses import dataclass

from pants.core.target_types import FileSourceField
from pants.core.util_rules.external_tool import DownloadedExternalTool, ExternalToolRequest
from pants.core.util_rules.source_files import SourceFiles, SourceFilesRequest
from pants.engine.addresses import Address, Addresses
from pants.engine.fs import Digest, MergeDigests
from pants.engine.platform import Platform
from pants.engine.rules import Get, MultiGet, collect_rules, rule
from pants.engine.target import (
    Dependencies,
    DependenciesRequest,
    InvalidTargetException,
    SourcesField,
    Targets,
)

from pants_backend_mdbook.subsystem import MdBookTool
from pants_backend_mdbook.targets import MdBookSources


@dataclass(frozen=True)
class MdBookAnalysis:
    digest: Digest
    build_root: str
    tool_exe: str


@dataclass(frozen=True)
class MdBookAnalysisRequest:
    address: Address


@rule(desc="Prepare MD Book Build Context")
async def prepare_md_book_ctx(
    request: MdBookAnalysisRequest, mdbook: MdBookTool, platform: Platform
) -> MdBookAnalysis:
    target = await Get(Targets, Addresses([request.address]))
    target = target[0]
    dependencies = await Get(Targets, DependenciesRequest(target[Dependencies]))

    (sources, codegened, tool) = await MultiGet(
        Get(
            SourceFiles,
            SourceFilesRequest(
                [target.get(MdBookSources)] + [t.get(SourcesField) for t in dependencies],
            ),
        ),
        Get(
            SourceFiles,
            SourceFilesRequest(
                [t.get(SourcesField) for t in dependencies],
                for_sources_types=(FileSourceField, MdBookSources),
                enable_codegen=True,
            ),
        ),
        Get(
            DownloadedExternalTool,
            ExternalToolRequest,
            mdbook.get_request(platform),
        ),
    )

    build_root = None
    for s in sources.files:
        if s.endswith("book.toml"):
            build_root = os.path.dirname(s)

    if build_root is None:
        raise InvalidTargetException("Must include a `book.toml` in the `md_book` sources.")

    sandbox_input = await Get(
        Digest, MergeDigests([tool.digest, sources.snapshot.digest, codegened.snapshot.digest])
    )
    return MdBookAnalysis(sandbox_input, build_root, tool.exe)


def rules():
    return collect_rules()



from pants_backend_mdbook import goals, subsystem, targets, util_rules


def target_types():
    return [
        *targets.targets(),
    ]


def rules():
    return [
        *subsystem.rules(),
        *util_rules.rules(),
        *goals.rules(),
    ]



"""

"""

from dataclasses import dataclass

from pants.core.goals.package import BuiltPackage, PackageFieldSet
from pants.engine.internals.selectors import Get
from pants.engine.rules import collect_rules, rule
from pants.engine.unions import UnionRule

from pants_backend_mdbook.targets import MdBookSources
from pants_backend_mdbook.util_rules.build import FallibleMdBookBuildOutput, MdbookBuildRequest


@dataclass(frozen=True)
class MdBookFieldSet(PackageFieldSet):
    required_fields = (MdBookSources,)

    sources: MdBookSources


@rule(desc="Package MDBOOK Image")
async def package_mdbook_image(field_set: MdBookFieldSet) -> BuiltPackage:
    build = await Get(FallibleMdBookBuildOutput, MdbookBuildRequest(field_set.address))
    if not build.success:
        raise Exception("Failed to build mdbook")

    assert build.output.digest is not None, "Expected a digest"
    return BuiltPackage(build.output.digest, tuple())


def rules():
    return [*collect_rules(), UnionRule(PackageFieldSet, MdBookFieldSet)]
